"use strict";

var P = require('bluebird');
var moment = require('moment');
var stringify = require('csv-stringify');
var _require = require('@forestadmin/context'),
  inject = _require.inject;
var ParamsFieldsDeserializer = require('../deserializers/params-fields');
var SmartFieldsValuesInjector = require('./smart-fields-values-injector');

// NOTICE: Prevent bad date formatting into timestamps.
var CSV_OPTIONS = {
  formatters: {
    date: function date(value) {
      return moment(value).format();
    }
  }
};
function CSVExporter(params, response, modelName, recordsExporter) {
  var _inject = inject(),
    configStore = _inject.configStore;
  this.perform = function () {
    var filename = "".concat(params.filename, ".csv");
    response.setHeader('Content-Type', 'text/csv; charset=utf-8');
    response.setHeader('Content-disposition', "attachment; filename=".concat(filename));
    response.setHeader('Last-Modified', moment());

    // NOTICE: From nginx doc: Setting this to "no" will allow unbuffered
    //         responses suitable for Comet and HTTP streaming applications.
    response.setHeader('X-Accel-Buffering', 'no');
    response.setHeader('Cache-Control', 'no-cache');
    var CSVHeader = "".concat(params.header, "\n");
    var CSVAttributes = params.fields[modelName].split(',');
    response.write(CSVHeader);
    var fieldsPerModel = new ParamsFieldsDeserializer(params.fields).perform();
    return recordsExporter.perform(function (records) {
      return P.map(records, function (record) {
        return new SmartFieldsValuesInjector(record, modelName, fieldsPerModel).perform();
      }).then(function (recordsWithSmartFieldsValues) {
        return new P(function (resolve) {
          if (configStore.Implementation.Flattener) {
            recordsWithSmartFieldsValues = configStore.Implementation.Flattener.flattenRecordsForExport(modelName, recordsWithSmartFieldsValues);
          }
          var CSVLines = [];
          recordsWithSmartFieldsValues.forEach(function (record) {
            var CSVLine = [];
            CSVAttributes.forEach(function (attribute) {
              var value;
              if (params.fields[attribute]) {
                if (record[attribute]) {
                  if (params.fields[attribute] && record[attribute][params.fields[attribute]]) {
                    value = record[attribute][params.fields[attribute]];
                  } else {
                    // eslint-disable-next-line
                    value = record[attribute].id || record[attribute]._id;
                  }
                }
              } else {
                value = record[attribute];
              }
              CSVLine.push(value || '');
            });
            CSVLines.push(CSVLine);
          });
          stringify(CSVLines, CSV_OPTIONS, function (error, csv) {
            response.write(csv);
            resolve();
          });
        });
      });
    }).then(function () {
      response.end();
    });
  };
}
module.exports = CSVExporter;